@@ -40,6 +40,8 @@ class Channel(DiscordObjectBase):
         self.member_count = resp.get("member_count")
         self.thread_metadata = ThreadMetadata.optional(self.client, resp.get("thread_metadata"))
         self.member = ThreadMember.optional(self.client, resp.get("member"))
+        self.default_auto_archive_duration = resp.get("default_auto_archive_duration")
+        self.permissions = resp.get("permissions")
 
     def create_message(self, *args, **kwargs):
         if not self.is_messageable():
@@ -138,6 +140,11 @@ class Channel(DiscordObjectBase):
     def list_joined_private_archived_threads(self, *, before: typing.Union[str, datetime.datetime] = None, limit: int = None):
         return self.client.list_joined_private_archived_threads(self, before, limit)
 
+    def archive(self, locked: bool = False):
+        if not self.is_thread_channel():
+            raise AttributeError("This type of channel is not allowed to archive.")
+        return self.modify(archived=True, locked=locked)
+
     @property
     def mention(self):
         return f"<#{self.id}>"
